<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                xmlns:xalan="http://xml.apache.org/xalan"
                xmlns:lav="http://software.in2p3.fr/lavoisier/config.xsd"
                xmlns="http://software.in2p3.fr/lavoisier/config.xsd"
                exclude-result-prefixes="xalan lav">
    <xsl:output method="xml" indent="yes" xalan:indent-amount="4"/>
    <xsl:strip-space elements="lav:*"/>
    <xsl:param name="validator.Disable">false</xsl:param>

    <!-- add default views -->
    <xsl:template match="/lav:config">
        <xsl:copy><xsl:apply-templates select="@*"/>
            <!-- operation example (optional) -->
            <view name="xsltproc">
                <info>
                    <category>Hidden</category>
                </info>
                <connector type="XSLTConnector">
                    <parameter name="stylesheet" eval="request()"/>
                    <parameter name="input">&lt;_/&gt;</parameter>
                    <parameter name="documents">
                        <xsl:variable name="console">|Monitoring|Configuration|Environment|Documentation|Hidden|</xsl:variable>
                        <xsl:for-each select="lav:view[@name][not(lav:authorized)][not(contains($console,concat('|',lav:info/lav:category/text(),'|')))]">
                            <entry key="{@name}" eval="view('{@name}')"/>
                        </xsl:for-each>
                    </parameter>
                </connector>
            </view>

            <!-- user defined views -->
            <xsl:apply-templates/>
        </xsl:copy>
    </xsl:template>

    <!-- replace default post-processors -->
    <xsl:template match="lav:view/@path-template">
        <xsl:copy-of select="."/>
        <xsl:if test="not(parent::lav:view/@post-processors)">
            <xsl:attribute name="post-processors">disable-default-processors</xsl:attribute>
        </xsl:if>
        <xsl:if test="parent::lav:view/lav:cache">
            <xsl:message terminate="yes">Not allowed to use cache adaptor on a view '<xsl:value-of
                    select="parent::lav:view/@name"/>' because it has some parameters.</xsl:message>
        </xsl:if>
    </xsl:template>

    <!-- add default renderer -->
    <xsl:template match="lav:view[not(lav:renderers)]">
        <xsl:copy><xsl:apply-templates select="@*"/>
            <xsl:apply-templates/>
            <renderers>
                <xsl:call-template name="RENDERERS"/>
            </renderers>
        </xsl:copy>
    </xsl:template>
    <xsl:template match="lav:renderers">
        <xsl:copy>
            <xsl:apply-templates/>
            <xsl:call-template name="RENDERERS"/>
        </xsl:copy>
    </xsl:template>
    <xsl:template match="lav:renderer[(@type='DefaultRenderer' or @type='HTMLRenderer' or @type='PDFRenderer') and not(lav:parameter[@name='stylesheet'])]">
        <xsl:copy><xsl:apply-templates select="@*"/>
            <xsl:apply-templates/>
            <xsl:choose>
                <xsl:when test="@type='DefaultRenderer'">
                    <xsl:call-template name="STYLESHEET_EXTERNAL"/>
                </xsl:when>
                <xsl:otherwise>
                    <xsl:call-template name="STYLESHEET_INTERNAL"/>
                </xsl:otherwise>
            </xsl:choose>
        </xsl:copy>
    </xsl:template>
    <xsl:template name="RENDERERS">
        <xsl:if test="not(lav:renderer[@type='DefaultRenderer'])">
            <renderer type="DefaultRenderer">
                <xsl:call-template name="STYLESHEET_EXTERNAL"/>
            </renderer>
        </xsl:if>
        <xsl:if test="not(lav:renderer[@type='HTMLRenderer'])">
            <renderer type="HTMLRenderer">
                <xsl:call-template name="STYLESHEET_INTERNAL"/>
            </renderer>
        </xsl:if>
        <xsl:if test="not(lav:renderer[@type='PDFRenderer'])">
            <renderer type="PDFRenderer">
                <xsl:call-template name="STYLESHEET_INTERNAL"/>
            </renderer>
        </xsl:if>
    </xsl:template>
    <xsl:template name="STYLESHEET_EXTERNAL">
        <parameter name="stylesheet">
            <xsl:choose>
                <xsl:when test="ancestor-or-self::lav:renderers/@html">/resource/<xsl:value-of select="ancestor-or-self::lav:renderers/@html"/></xsl:when>
                <xsl:when test="ancestor-or-self::lav:view/@name">/lavoisier/xsl/<xsl:value-of select="ancestor-or-self::lav:view/@name"/></xsl:when>
                <xsl:otherwise>/lavoisier/xsl/_default_</xsl:otherwise>
            </xsl:choose>
        </parameter>
    </xsl:template>
    <xsl:template name="STYLESHEET_INTERNAL">
        <parameter name="stylesheet">
            <xsl:choose>
                <xsl:when test="ancestor-or-self::lav:renderers/@html"><xsl:value-of select="ancestor-or-self::lav:renderers/@html"/></xsl:when>
                <xsl:when test="ancestor-or-self::lav:view/@name">memory://<xsl:value-of select="ancestor-or-self::lav:view/@name"/></xsl:when>
                <xsl:otherwise>memory://_default_</xsl:otherwise>
            </xsl:choose>
        </parameter>
    </xsl:template>

    <!-- disable XML validation -->
    <xsl:template match="lav:processor[contains(@type, 'Validator') and $validator.Disable='true']"/>

    <!-- add default indexed cache parameter -->
    <xsl:template match="lav:cache[@type='IndexedFileCache' and not(lav:parameter/@name='xpath')]">
        <xsl:copy><xsl:apply-templates select="@*"/>
            <xsl:apply-templates select="lav:parameter"/>
            <xsl:comment> generated automatically </xsl:comment>
            <parameter name="xpath" eval="path('')"/>
            <xsl:apply-templates select="lav:trigger"/>
        </xsl:copy>
    </xsl:template>

    <!-- clone merge processor -->
    <xsl:template match="lav:merge[@count]">
        <xsl:param name="count" select="number(@count)"/>
        <xsl:if test="$count > 0">
            <xsl:copy>
                <xsl:apply-templates select="@*[not(local-name()='count')] | * | text() | comment()"/>
            </xsl:copy>
            <xsl:apply-templates select="self::lav:merge">
                <xsl:with-param name="count" select="$count - 1"/>
            </xsl:apply-templates>
        </xsl:if>
    </xsl:template>

    <!-- default rules -->
    <xsl:template match="@*">
        <xsl:copy-of select="."/>
    </xsl:template>
    <xsl:template match="*">
        <xsl:param name="select"/>
        <xsl:copy>
            <xsl:apply-templates select="@*|*|text()|comment()">
                <xsl:with-param name="select" select="$select"/>
            </xsl:apply-templates>
        </xsl:copy>
    </xsl:template>
</xsl:stylesheet>