<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                xmlns:xalan="http://xml.apache.org/xalan"
                xmlns:lav="http://software.in2p3.fr/lavoisier/config.xsd"
                xmlns="http://software.in2p3.fr/lavoisier/config.xsd"
                exclude-result-prefixes="xalan lav">
    <xsl:output method="xml" indent="yes" xalan:indent-amount="4"/>
    <xsl:strip-space elements="lav:*"/>

    <xsl:template match="lav:processors/lav:*[not(self::lav:processor|self::lav:for)]">
        <processor>
            <xsl:attribute name="type">
                <xsl:choose>
                    <xsl:when test="self::lav:aggregate">AppendAggregateProcessor</xsl:when>
                    <xsl:when test="self::lav:insert and @as='parent'">InsertAsParentProcessor</xsl:when>
                    <xsl:when test="self::lav:insert and not(@as='parent')">InsertProcessor</xsl:when>
                    <xsl:when test="self::lav:merge">MergeProcessor</xsl:when>
                    <xsl:when test="self::lav:move">MoveProcessor</xsl:when>
                    <xsl:when test="self::lav:remove">RemoveProcessor</xsl:when>
                    <xsl:when test="self::lav:replace">ReplaceProcessor</xsl:when>
                    <xsl:when test="self::lav:select and lav:group">GroupProcessor</xsl:when>
                    <xsl:when test="self::lav:select and lav:if-descendant">SelectAncestorProcessor</xsl:when>
                    <xsl:when test="self::lav:select and not(lav:*)">SelectProcessor</xsl:when>
                    <xsl:when test="self::lav:unique">UniqueProcessor</xsl:when>
                    <xsl:otherwise><xsl:message terminate="yes">Processor not supported: <xsl:value-of
                            select="local-name()"/></xsl:message></xsl:otherwise>
                </xsl:choose>
            </xsl:attribute>
            <parameter name="namespaces">
                <xsl:for-each select="ancestor::lav:view/lav:namespace">
                    <entry key="{@prefix}"><xsl:value-of select="@uri"/></entry>
                </xsl:for-each>
            </parameter>
            <parameter name="xml_variables">
                <xsl:if test="(@* | */@*)[contains(., 'current()')]">
                    <entry key="current" eval="request()"/>
                </xsl:if>
            </parameter>
            <xsl:apply-templates mode="pixtl" select="@* | */@* | lav:to-following | lav:to-ancestor"/>
        </processor>
    </xsl:template>

    <xsl:template mode="pixtl" match="lav:insert[@as='parent']/@as"/>
    <xsl:template mode="pixtl" match="lav:to-following/@sibling"/>
    <xsl:template mode="pixtl" match="lav:to-following[@sibling='true']">
        <parameter name="destination_axis">following-sibling</parameter>
    </xsl:template>
    <xsl:template mode="pixtl" match="lav:to-following">
        <parameter name="destination_axis">following</parameter>
    </xsl:template>
    <xsl:template mode="pixtl" match="lav:to-ancestor">
        <parameter name="destination_axis">ancestor</parameter>
    </xsl:template>

    <xsl:template mode="pixtl" match="@*">
        <parameter>
            <xsl:attribute name="name"><xsl:apply-templates mode="name" select="."/></xsl:attribute>
            <xsl:call-template name="CURRENT">
                <xsl:with-param name="xpath" select="."/>
            </xsl:call-template>
        </parameter>
    </xsl:template>

    <xsl:template name="CURRENT">
        <xsl:param name="xpath"/>
        <xsl:choose>
            <xsl:when test="contains($xpath, 'current()')">
                <xsl:value-of select="substring-before($xpath, 'current()')"/>$current<xsl:text/>
                <xsl:call-template name="CURRENT">
                    <xsl:with-param name="xpath" select="substring-after($xpath, 'current()')"/>
                </xsl:call-template>
            </xsl:when>
            <xsl:otherwise>
                <xsl:value-of select="$xpath"/>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>

    <xsl:template mode="name" match="lav:*/@nodes">nodes</xsl:template>
    <xsl:template mode="name" match="lav:unique/@values">nodes</xsl:template>
    <xsl:template mode="name" match="lav:aggregate/@context | lav:insert/@context">nodes</xsl:template>

    <xsl:template mode="name" match="lav:aggregate/@function">function</xsl:template>
    <xsl:template mode="name" match="lav:group/@by">group_by</xsl:template>
    <xsl:template mode="name" match="lav:group/@context-depth">context_depth</xsl:template>
    <xsl:template mode="name" match="lav:group/@sort | lav:unique/@sort">sort_type</xsl:template>
    <xsl:template mode="name" match="lav:group/@descending | lav:unique/@descending">sort_descending</xsl:template>
    <xsl:template mode="name" match="lav:select/@depth | lav:remove/@depth">depth</xsl:template>
    <xsl:template mode="name" match="lav:select/@single">single_node</xsl:template>

    <xsl:template mode="name" match="lav:node/@prefix | lav:element/@prefix">node_prefix</xsl:template>
    <xsl:template mode="name" match="lav:node/@name | lav:element/@name">node_name</xsl:template>
    <xsl:template mode="name" match="lav:node/@name_as_xpath">node_name_as_xpath</xsl:template>
    <xsl:template mode="name" match="lav:node/@value">node_value</xsl:template>
    <xsl:template mode="name" match="lav:element/@values">node_values</xsl:template>
    <xsl:template mode="name" match="lav:to-following/@as | lav:to-ancestor/@as | lav:insert/@as">destination_as</xsl:template>
    <xsl:template mode="name" match="lav:to-following/@prefix | lav:to-ancestor/@prefix">destination_prefix</xsl:template>
    <xsl:template mode="name" match="lav:to-following/@name | lav:to-ancestor/@name">destination_name</xsl:template>
    <xsl:template mode="name" match="lav:to-following/@predicate | lav:to-ancestor/@predicate">destination_predicate</xsl:template>
    <xsl:template mode="name" match="lav:if-descendant/@prefix">if_descendant_prefix</xsl:template>
    <xsl:template mode="name" match="lav:if-descendant/@name">if_descendant_name</xsl:template>
    <xsl:template mode="name" match="lav:if-descendant/@predicate">if_descendant_predicate</xsl:template>

    <!-- default rules -->
    <xsl:template match="@*">
        <xsl:copy-of select="."/>
    </xsl:template>
    <xsl:template match="*">
        <xsl:copy>
            <xsl:apply-templates select="@*|*|text()|comment()"/>
        </xsl:copy>
    </xsl:template>
</xsl:stylesheet>