<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform">
    <xsl:output method="xml" indent="yes"/>

    <xsl:template match="/XPath">
        <xsl:for-each select="descendant::RelativeLocationPath[not(
                                ancestor::AbsoluteLocationPath or
                                ancestor-or-self::RelativeLocationPath/preceding-sibling::FilterExpr/VariableReference
                              )]">
            <xsl:message terminate="yes">Relative path not supported in this context: <xsl:apply-templates select="."/></xsl:message>
        </xsl:for-each>
        <xsl:for-each select="descendant::*[local-name()!='PathExpr' and local-name()!='UnionExpr' and contains(local-name(),'Expr')][not(
                                ancestor::AbsoluteLocationPath or
                                ancestor-or-self::RelativeLocationPath/preceding-sibling::FilterExpr/VariableReference or
                                self::FilterExpr/child::VariableReference
                              )]">
            <xsl:message terminate="yes">Expression not supported in this context: <xsl:apply-templates select="."/></xsl:message>
        </xsl:for-each>
        <xsl:choose>
            <xsl:when test="PathExpr/FilterExpr/VariableReference">
                <straxpath>
                    <xsl:attribute name="relative">
                        <xsl:apply-templates/>
                    </xsl:attribute>
                </straxpath>
            </xsl:when>
            <xsl:when test="descendant::AbsoluteLocationPath/descendant::Function
                            [@s1='position' or @s1='last']
                            [not(ancestor::RelativeLocationPath/preceding-sibling::FilterExpr/VariableReference)]
                          | descendant::Predicate[PathExpr/FilterExpr/Number/@i][count(*)=1][count(*/*)=1][count(*/*/*)=1]
                          | descendant::AbsoluteLocationPath/descendant::NameStep
                            [   starts-with(@axis,'preceding')
                                or starts-with(@axis,'following')
                                or ( (starts-with(@axis,'ancestor') or @axis='parent') and
                                     (not(ancestor::Predicate) or following-sibling::NameStep/@axis='child' or following-sibling::TextNodeStep or following-sibling::CommentNodeStep)
                                   )
                            ]
                            [not(ancestor::RelativeLocationPath/preceding-sibling::FilterExpr/VariableReference)]
                          | descendant::VariableReference[not(ancestor::AbsoluteLocationPath)]
            ">
                <straxpath>
                    <xsl:attribute name="relative">
                        <xsl:apply-templates/>
                    </xsl:attribute>
                </straxpath>
            </xsl:when>
            <xsl:otherwise>
                <straxpath>
                    <xsl:apply-templates select="(PathExpr | descendant::UnionExpr[not(ancestor::Predicate)]/PathExpr)/AbsoluteLocationPath" mode="path"/>
                </straxpath>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>

    <!-- path -->
    <xsl:template match="AbsoluteLocationPath" mode="path">
        <xsl:choose>
            <xsl:when test="descendant::Predicate[descendant::NameStep[@axis='child' or @axis='descendant']|descendant::TextNodeStep|descendant::CommentNodeStep]">
                <xsl:apply-templates mode="path" select="*
                           [descendant::Predicate[descendant::NameStep[@axis='child' or @axis='descendant']|descendant::TextNodeStep|descendant::CommentNodeStep]][1]"/>
            </xsl:when>
            <xsl:otherwise>
                <xsl:apply-templates select="*[last()]" mode="path"/>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>

    <xsl:template match="NameStep | TextNodeStep | CommentNodeStep" mode="path">
        <xsl:variable name="tag">
            <xsl:choose>
                <xsl:when test="self::NameStep and @axis!='attribute'">element</xsl:when>
                <xsl:when test="self::NameStep and @axis='attribute'">attribute</xsl:when>
                <xsl:when test="self::TextNodeStep">text</xsl:when>
                <xsl:when test="self::CommentNodeStep">comment</xsl:when>
            </xsl:choose>
        </xsl:variable>
        <xsl:element name="{$tag}">
            <xsl:copy-of select="@prefix|@localName"/>
            <xsl:if test="self::NameStep/following-sibling::*">
                <xsl:attribute name="select">.<xsl:apply-templates select="self::NameStep/following-sibling::*"/></xsl:attribute>
            </xsl:if>
            <xsl:if test="boolean(preceding-sibling::*) or @axis='child'">
                <predicate requiresDescendants="false">
                    <xsl:text>.</xsl:text>
                    <xsl:apply-templates select="preceding-sibling::*">
                        <xsl:with-param name="reverse">true</xsl:with-param>
                        <xsl:sort select="position()" order="descending" data-type="number"/>
                    </xsl:apply-templates>
                    <xsl:if test="parent::*/NameStep[1][@axis='child']">[not(ancestor::*)]</xsl:if>
                </predicate>
            </xsl:if>
            <xsl:apply-templates select="*" mode="path">
                <xsl:sort select="count(descendant::NameStep[@axis='child' or @axis='descendant']|descendant::TextNodeStep|descendant::CommentNodeStep)"/>
            </xsl:apply-templates>
        </xsl:element>
    </xsl:template>
    <xsl:template match="AllNodeStep" mode="path">
        <xsl:message terminate="yes">StrAXPath does not support '//*', please use '/descendant-or-self::*' instead.</xsl:message>
    </xsl:template>

    <xsl:template match="Predicate" mode="path">
        <predicate requiresDescendants="{boolean(
            (descendant::NameStep[@axis='child' or @axis='descendant']|descendant::TextNodeStep|descendant::CommentNodeStep)
            [not(ancestor::RelativeLocationPath/preceding-sibling::FilterExpr/VariableReference)]
        )}">
            <xsl:apply-templates select="*[last()]"/>
        </predicate>
    </xsl:template>

    <xsl:template match="*" mode="path">
        <xsl:message terminate="yes">[INTERNAL ERROR] unexpected node: <xsl:value-of select="local-name()"/></xsl:message>
    </xsl:template>

    <xsl:include href="xpath-serializer.xsl"/>
</xsl:stylesheet>
