<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
        xmlns:cfg="http://software.in2p3.fr/lavoisier/config.xsd">
    <xsl:output method="text"/>

<xsl:template match="/">digraph G {
    style=dashed;
    <xsl:apply-templates/>
}
</xsl:template>

    <xsl:template match="cfg:view">subgraph "cluster_<xsl:value-of select="@name"/>" {
        label = "<xsl:value-of select="@name"/>";
        tooltip = "<xsl:value-of select="cfg:info/cfg:description/text()"/>";
        <xsl:apply-templates select="cfg:*[not(self::cfg:info|self::cfg:authorized|self::cfg:pre-renderers|self::cfg:renderers|self::cfg:exception|self::cfg:fallback)]"/>
        <xsl:text>    </xsl:text>
        <xsl:for-each select="descendant::cfg:connector|descendant::cfg:serializer|descendant::cfg:processor|descendant::cfg:cache">
            <xsl:text/>"<xsl:apply-templates select="." mode="node"/>"<xsl:text/>
            <xsl:choose>
                <xsl:when test="not(position()=last())"> -> </xsl:when>
                <xsl:otherwise>;</xsl:otherwise>
            </xsl:choose>
        </xsl:for-each>
    }
    </xsl:template>

    <xsl:template match="cfg:for">
        <xsl:text/>    subgraph "cluster_<xsl:value-of select="parent::cfg:view/@name"/>_FOR" {
            label="for";
            style=rounded;
            <xsl:apply-templates select="cfg:*"/>
        }
    </xsl:template>

    <xsl:template match="cfg:connector|cfg:serializer|cfg:processor|cfg:cache">
        <xsl:variable name="class_name">
            <xsl:call-template name="CLASS_NAME">
                <xsl:with-param name="string" select="@type"/>
            </xsl:call-template>
        </xsl:variable>
        <xsl:variable name="suffix">
            <xsl:apply-templates select="." mode="suffix"/>
        </xsl:variable>
        <xsl:variable name="label" select="substring-before($class_name, $suffix)"/>

        <xsl:text>    "</xsl:text>
        <xsl:apply-templates select="." mode="node"/>
        <xsl:text>" [</xsl:text>
        <xsl:choose>
            <xsl:when test="(self::cfg:connector or self::cfg:cache) and string-length($label)>10">
                <xsl:text/>label="<xsl:value-of select="substring($label, 0, 7)"/>..."<xsl:text/>
            </xsl:when>
            <xsl:otherwise>label="<xsl:value-of select="$label"/>"</xsl:otherwise>
        </xsl:choose>
        <xsl:text/>, tooltip="<xsl:value-of select="$label"/>"<xsl:text/>
        <xsl:text/>, shape=<xsl:apply-templates select="." mode="shape"/>];
    </xsl:template>

    <xsl:template match="cfg:connector|cfg:serializer|cfg:processor|cfg:cache" mode="node">
        <xsl:value-of select="ancestor::cfg:view/@name"/>
        <xsl:if test="parent::cfg:for">_FOR</xsl:if>
        <xsl:text/>_<xsl:value-of select="count(preceding-sibling::cfg:*)+1"/>
    </xsl:template>

    <xsl:template match="cfg:connector" mode="suffix">Connector</xsl:template>
    <xsl:template match="cfg:serializer" mode="suffix">Serializer</xsl:template>
    <xsl:template match="cfg:processor" mode="suffix">Processor</xsl:template>
    <xsl:template match="cfg:cache" mode="suffix">Cache</xsl:template>

    <xsl:template match="cfg:connector" mode="shape">circle, width=.75, fixedsize=true</xsl:template>
    <xsl:template match="cfg:serializer" mode="shape">house</xsl:template>
    <xsl:template match="cfg:processor" mode="shape">box</xsl:template>
    <xsl:template match="cfg:cache" mode="shape">doublecircle, width=.75, fixedsize=true</xsl:template>

    <xsl:template name="CLASS_NAME">
        <xsl:param name="string"/>
        <xsl:choose>
            <xsl:when test="contains($string, '.')">
                <xsl:call-template name="CLASS_NAME">
                    <xsl:with-param name="string" select="substring-after($string, '.')"/>
                </xsl:call-template>
            </xsl:when>
            <xsl:otherwise>
                <xsl:value-of select="$string"/>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>
</xsl:stylesheet>