<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform">
    <xsl:output method="text"/>

    <xsl:template match="NameStep">
        <xsl:param name="reverse">false</xsl:param>
        <xsl:if test="position()>1 or parent::AbsoluteLocationPath or parent::RelativeLocationPath/preceding-sibling::*[1]/self::FilterExpr[VariableReference|Function]">/</xsl:if>
        <xsl:choose>
            <xsl:when test="$reverse='false'"><xsl:value-of select="@axis"/></xsl:when>
            <xsl:when test="following-sibling::NameStep[1]/@axis='attribute'">parent</xsl:when>
            <xsl:when test="following-sibling::NameStep[1]/@axis='child'">parent</xsl:when>
            <xsl:when test="following-sibling::NameStep[1]/@axis='descendant'">ancestor</xsl:when>
            <xsl:when test="following-sibling::NameStep[1]/@axis='descendant-or-self'">ancestor-or-self</xsl:when>
            <xsl:when test="following-sibling::NameStep[1]/@axis='self'">self</xsl:when>
            <xsl:when test="following-sibling::TextNodeStep[1]">parent</xsl:when>
            <xsl:when test="following-sibling::CommentNodeStep[1]">parent</xsl:when>
            <xsl:otherwise>
                <xsl:message terminate="yes">INTERNAL ERROR (unexpected node): <xsl:value-of select="local-name()"/></xsl:message>
            </xsl:otherwise>
        </xsl:choose>
        <xsl:text/>::<xsl:if test="@prefix"><xsl:value-of select="@prefix"/>:</xsl:if><xsl:value-of select="@localName"/>
        <xsl:apply-templates select="Predicate"/>
    </xsl:template>
    <xsl:template match="TextNodeStep">
        <xsl:if test="not(parent::RelativeLocationPath and position()=1)">/</xsl:if>
        <xsl:text>text()</xsl:text>
        <xsl:apply-templates select="Predicate"/>
    </xsl:template>
    <xsl:template match="CommentNodeStep">
        <xsl:if test="not(parent::RelativeLocationPath and position()=1)">/</xsl:if>
        <xsl:text>comment()</xsl:text>
        <xsl:apply-templates select="Predicate"/>
    </xsl:template>
    <xsl:template match="AllNodeStep">
        <xsl:if test="not(parent::RelativeLocationPath and position()=1)">/</xsl:if>
        <xsl:choose>
            <xsl:when test="@axis='self'">.</xsl:when>
            <xsl:otherwise><xsl:value-of select="@axis"/>::*<xsl:text/></xsl:otherwise>
        </xsl:choose>
        <xsl:apply-templates select="Predicate"/>
    </xsl:template>
    <xsl:template match="ProcessingInstructionNodeStep">&lt;?<xsl:value-of select="@s"/>?&gt;</xsl:template>

    <xsl:template match="Function">
        <xsl:value-of select="@s1"/>(<xsl:for-each select="
                (PathExpr|UnaryExpr|UnionExpr|OrExpr|AndExpr)[not(following-sibling::*[1][not(contains('PathExpr|UnaryExpr|UnionExpr|OrExpr|AndExpr',local-name()))])]
              | *[not(contains('PathExpr|UnaryExpr|UnionExpr|OrExpr|AndExpr',local-name()))][following-sibling::*[1][contains('PathExpr|UnaryExpr|UnionExpr|OrExpr|AndExpr',local-name())] or position()=last()]
        ">
            <xsl:if test="position()>1">,</xsl:if><xsl:apply-templates select="."/>
        </xsl:for-each>)<xsl:text/>
    </xsl:template>
    <xsl:template match="UnionExpr | OrExpr | AndExpr">
        <xsl:for-each select="
                (PathExpr|UnaryExpr|UnionExpr|OrExpr|AndExpr)[not(following-sibling::*[1][not(contains('PathExpr|UnaryExpr|UnionExpr|OrExpr|AndExpr',local-name()))])]
              | *[not(contains('PathExpr|UnaryExpr|UnionExpr|OrExpr|AndExpr',local-name()))][following-sibling::*[1][contains('PathExpr|UnaryExpr|UnionExpr|OrExpr|AndExpr',local-name())] or position()=last()]
        ">
            <xsl:if test="position()>1"><xsl:value-of select="parent::*/@symbol"/></xsl:if><xsl:apply-templates select="."/>
        </xsl:for-each>
    </xsl:template>
    <xsl:template match="PathExpr">
        <xsl:apply-templates select="*"/>
    </xsl:template>
    <xsl:template match="UnaryExpr">
        <xsl:value-of select="@symbol"/><xsl:apply-templates select="*"/>
    </xsl:template>

    <!-- symbol - BEGIN -->
    <xsl:template match="Function" mode="symbol">,</xsl:template>
    <xsl:template match="UnionExpr | OrExpr | AndExpr" mode="symbol"><xsl:value-of select="@symbol"/></xsl:template>
    <xsl:template match="*" mode="symbol"/>
    <!-- symbol - END -->

    <xsl:template match="Number"><xsl:value-of select="@i"/></xsl:template>
    <xsl:template match="Literal">'<xsl:value-of select="@s"/>'</xsl:template>
    <xsl:template match="VariableReference">$<xsl:value-of select="@s1"/></xsl:template>

    <xsl:template match="Predicate">[<xsl:apply-templates select="*[last()]"/>]</xsl:template>
    <xsl:template match="*[local-name()!='OrExpr' and local-name()!='AndExpr' and local-name()!='UnionExpr' and local-name()!='UnaryExpr'][@symbol]">
        <xsl:apply-templates select="preceding-sibling::*[1]"/>
        <xsl:value-of select="@symbol"/>
        <xsl:apply-templates select="*[last()]"/>
    </xsl:template>

    <xsl:template match="*">
        <xsl:apply-templates select="*"/>
    </xsl:template>
</xsl:stylesheet>