<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform">
    <xsl:output method="xml" indent="yes"/>

    <xsl:template match="/XPath">
        <xsl:if test="*[not(self::PathExpr)] |
                      PathExpr/*[not(self::AbsoluteLocationPath)] |
                      PathExpr/AbsoluteLocationPath/*[1][not(self::NameStep[@axis='child'])] |
                      descendant::NameStep[@axis!='child' and @axis!='attribute'] |
                      descendant::RelationalExpr |
                      descendant::EqualityExpr[@operator='Differs' or not(PathExpr/FilterExpr/Literal)] |
                      descendant::Function">
            <xsl:message terminate="yes">Unsupported XPath expression</xsl:message>
        </xsl:if>
        <xsl:apply-templates select="PathExpr/AbsoluteLocationPath/NameStep[1]"/>
    </xsl:template>

    <xsl:template match="/XPath/PathExpr/AbsoluteLocationPath/NameStep/Predicate">
        <xsl:variable name="predicate" select="."/>
        <xsl:for-each select="descendant::OrExpr">
            <xsl:apply-templates select="$predicate/*">
                <xsl:with-param name="alternative" select="*[not(position()=last())][not(following-sibling::*[1][self::EqualityExpr])]"/>
            </xsl:apply-templates>
            <xsl:apply-templates select="$predicate/*">
                <xsl:with-param name="alternative" select="*[last()]"/>
            </xsl:apply-templates>
        </xsl:for-each>
    </xsl:template>
    <xsl:template match="OrExpr">
        <xsl:param name="alternative"/>
        <xsl:apply-templates select="$alternative"/>
    </xsl:template>

    <xsl:template match="NameStep[@axis='child']">
        <xsl:param name="alternative"/>
        <xsl:element name="{@localName}">
            <xsl:apply-templates select="Predicate">
                <xsl:with-param name="alternative" select="$alternative"/>
            </xsl:apply-templates>
            <xsl:apply-templates select="following-sibling::*[1]"/>
        </xsl:element>
    </xsl:template>
    <xsl:template match="NameStep[@axis='attribute']">
        <xsl:param name="value"/>
        <xsl:attribute name="{@localName}">
            <xsl:value-of select="$value"/>
        </xsl:attribute>
    </xsl:template>

    <xsl:template match="AndExpr">
        <xsl:param name="alternative"/>
        <xsl:apply-templates select="*[not(position()=last())][not(following-sibling::*[1][self::EqualityExpr])]">
            <xsl:with-param name="alternative" select="$alternative"/>
        </xsl:apply-templates>
        <xsl:apply-templates select="*[last()]">
            <xsl:with-param name="alternative" select="$alternative"/>
        </xsl:apply-templates>
    </xsl:template>

    <xsl:template match="EqualityExpr[PathExpr/FilterExpr/Literal]">
        <xsl:apply-templates select="preceding-sibling::*[1]">
            <xsl:with-param name="value">
                <xsl:value-of select="PathExpr/FilterExpr/Literal/@s"/>
            </xsl:with-param>
        </xsl:apply-templates>
    </xsl:template>

    <xsl:template match="*">
        <xsl:param name="value"/>
        <xsl:param name="alternative"/>
        <xsl:apply-templates select="*">
            <xsl:with-param name="value" select="$value"/>
            <xsl:with-param name="alternative" select="$alternative"/>
        </xsl:apply-templates>
    </xsl:template>
</xsl:stylesheet>