<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform">
    <xsl:output method="xml" indent="yes" xalan:indent-amount="4" xmlns:xalan="http://xml.apache.org/xslt"/>
    <xsl:strip-space elements="*"/>

    <xsl:template match="/*/*/*[not(self::AbsoluteLocationPath)]">
        <xsl:message terminate="yes">This adaptor only supports absolute path</xsl:message>
    </xsl:template>
    <xsl:template match="NameStep[@axis!='child']">
        <xsl:message terminate="yes">This adaptor only supports child axis</xsl:message>
    </xsl:template>
    <xsl:template match="Predicate[ancestor::Predicate]">
        <xsl:message terminate="yes">This adaptor does not support nested predicates</xsl:message>
    </xsl:template>
    <xsl:template match="AbsoluteLocationPath[ancestor::Predicate]">
        <xsl:message terminate="yes">This adaptor does not support absolute path in predicates</xsl:message>
    </xsl:template>
    <xsl:template match="Function[@s1!='not']">
        <xsl:message terminate="yes">This adaptor does not yet support functions (please let us know if you need it)</xsl:message>
    </xsl:template>
    <xsl:template match="AllNodeStep">
        <xsl:message terminate="yes">This adaptor does not support unnamed node</xsl:message>
    </xsl:template>
    <xsl:template match="CommentNodeStep | ProcessingInstructionNodeStep | UnionExpr | VariableReference">
        <xsl:message terminate="yes">This adaptor does not support <xsl:value-of select="local-name()"/></xsl:message>
    </xsl:template>

    <xsl:template match="AbsoluteLocationPath | RelativeLocationPath">
        <xsl:copy>
            <xsl:apply-templates/>
        </xsl:copy>
    </xsl:template>

    <xsl:template match="NameStep[@axis='child']">
        <xsl:copy>
            <xsl:copy-of select="@prefix | @localName"/>
            <xsl:apply-templates/>
        </xsl:copy>
    </xsl:template>

    <xsl:template match="Predicate">
        <xsl:apply-templates select="*[last()]"/>
    </xsl:template>
    <xsl:template match="AndExpr | OrExpr">
        <xsl:copy>
            <xsl:apply-templates select="(PathExpr|AndExpr|OrExpr|NotExpr)[last()]/preceding-sibling::*[1]"/>
            <xsl:apply-templates select="*[last()]"/>
        </xsl:copy>
    </xsl:template>
    <xsl:template match="Function[@s1='not']">
        <NotExpr>
            <xsl:apply-templates select="*[last()]"/>
        </NotExpr>
    </xsl:template>

    <xsl:template match="EqualityExpr | RelationalExpr">
        <Comparison>
            <xsl:copy-of select="@symbol"/>
            <xsl:apply-templates select="preceding-sibling::*[1]"/>
            <xsl:apply-templates select="*"/>
        </Comparison>
    </xsl:template>
    <xsl:template match="UnaryExpr">
        <xsl:copy>
            <xsl:copy-of select="@symbol"/>
            <xsl:apply-templates select="*"/>
        </xsl:copy>
    </xsl:template>

    <xsl:template match="Literal">
        <xsl:copy>
            <xsl:value-of select="@s"/>
        </xsl:copy>
    </xsl:template>
    <xsl:template match="Number">
        <xsl:copy>
            <xsl:value-of select="@i"/>
        </xsl:copy>
    </xsl:template>
</xsl:stylesheet>