<!-- others.xsl -->
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                xmlns:e="http://software.in2p3.fr/lavoisier/entries.xsd"
                xmlns:cfg="http://software.in2p3.fr/lavoisier/config.xsd">
    <xsl:output method="html"/>

    <xsl:variable name="confView" select="document('conf_view')"/>
    <xsl:variable name="statusView" select="document('status_view')"/>
    <xsl:variable name="mimeView" select="document('mime_view')"/>
    <xsl:variable name="navbarView" select="document('navbar_view')"/>
    <xsl:variable name="headView" select="document('head_view')"/>
    <xsl:key name="cat_id" match="cfg:category" use="."/>
    <xsl:variable name="defaultCatLabel" select="'Default category'"/>

    <xsl:template match="/">
        <html>
            <xsl:copy-of select="$headView"/>
            <body data-spy="scroll" data-target=".bs-docs-sidebar" data-twttr-rendered="true">
                <xsl:copy-of select="$navbarView"/>
                <xsl:variable name="catMatch"
                              select="$confView/cfg:config/cfg:view[not(cfg:info/cfg:category/text() = /menu/item/@category or cfg:info/cfg:category/text() = 'Hidden' or cfg:info/cfg:category/text() = false()) or not(cfg:info/cfg:category)]"/>
                <xsl:variable name="catNoMatch"
                              select="$confView/cfg:config/cfg:view[not(cfg:info/cfg:category/text() = /menu/item/@category or cfg:info/cfg:category/text() = 'Hidden') and (not(cfg:info/cfg:category) or not(cfg:info/cfg:category/text()))]"/>
                <div class="container fluid">
                    <div class="row">
                        <div class="span3 bs-docs-sidebar">
                            <ul class="nav nav-list bs-docs-sidenav affix">
                                <xsl:if test="$catNoMatch">
                                    <xsl:call-template name="menuItem">
                                        <xsl:with-param name="itemName" select="$defaultCatLabel"/>
                                    </xsl:call-template>
                                </xsl:if>
                                <xsl:apply-templates select="$catMatch" mode="catList"/>
                            </ul>
                        </div>
                        <div class="span9">
                            <xsl:apply-templates select="$catMatch" mode="catView"/>
                            <xsl:if test="$catNoMatch">
                                <section>
                                    <xsl:attribute name="id">
                                        <xsl:value-of select="$defaultCatLabel"/>
                                    </xsl:attribute>
                                    <div class="page-header">
                                        <h1>
                                            <small>
                                                <xsl:value-of select="$defaultCatLabel"/>
                                            </small>
                                        </h1>
                                    </div>
                                    <table class="table table-bordered table-striped">
                                        <xsl:call-template name="tHead"/>
                                        <tbody>
                                            <xsl:for-each select="$catNoMatch">
                                                <xsl:apply-templates select="." mode="viewRow"/>
                                            </xsl:for-each>
                                        </tbody>
                                    </table>
                                </section>
                            </xsl:if>
                        </div>
                    </div>
                </div>
            </body>
        </html>
    </xsl:template>

    <!-- left category menu -->
    <xsl:template match="cfg:view" mode="catList">
        <xsl:if test="(generate-id(cfg:info/cfg:category)=generate-id(key('cat_id', cfg:info/cfg:category)[1]) and cfg:info/cfg:category and cfg:info/cfg:category/text()) ">
            <xsl:call-template name="menuItem">
                <xsl:with-param name="itemName" select="cfg:info/cfg:category/text()"/>
            </xsl:call-template>
        </xsl:if>
    </xsl:template>

    <!-- left category item menu -->
    <xsl:template name="menuItem">
        <xsl:param name="itemName"/>
        <li>
            <a>
                <xsl:attribute name="href">
                    #<xsl:value-of select="$itemName"/>
                </xsl:attribute>
                <!--<i class="icon-chevron-right"></i>-->
                <xsl:value-of select="$itemName"/>
            </a>
        </li>
    </xsl:template>

    <!-- category & view list section -->
    <xsl:template match="cfg:view" mode="catView">
        <xsl:if test="(generate-id(cfg:info/cfg:category)=generate-id(key('cat_id', cfg:info/cfg:category)[1]) and cfg:info/cfg:category and cfg:info/cfg:category/text() )">
            <xsl:variable name="catName" select="cfg:info/cfg:category/text()"/>
            <section>
                <xsl:attribute name="id">
                    <xsl:value-of select="$catName"/>
                </xsl:attribute>
                <div class="page-header">
                    <h2>
                        <xsl:value-of select="$catName"/>
                    </h2>
                </div>
                <table class="table table-bordered table-striped">
                    <xsl:call-template name="tHead"/>
                    <tbody>
                        <xsl:apply-templates
                                select="$confView/cfg:config/cfg:view[cfg:info/cfg:category/text() = $catName]"
                                mode="viewRow"/>
                    </tbody>
                </table>
            </section>
        </xsl:if>
    </xsl:template>

    <!-- table head -->
    <xsl:template name="tHead">
        <thead>
            <tr>
                <th>name</th>
                <th>status</th>
                <th>description</th>
                <th>path</th>
                <th>rendering</th>
                <th>configuration</th>
            </tr>
        </thead>
    </xsl:template>

    <!--  view as a row -->
    <xsl:template match="cfg:view" mode="viewRow">

        <!-- if view not already treated -->
        <xsl:variable name="viewName" select="@name"/>
        <xsl:variable name="statusNode" select="$statusView/jmx/domain/group/mbean[@name=$viewName]"/>
        <tr>
            <td>
                <a title="see view with default rendering">
                    <xsl:attribute name="href">
                        /lavoisier/<xsl:value-of select="$viewName"/>
                    </xsl:attribute>
                    <xsl:choose>
                        <xsl:when test="not(cfg:info/cfg:label/text())">
                            <xsl:value-of select="$viewName"/>
                        </xsl:when>
                        <xsl:otherwise>
                            <xsl:value-of select="cfg:info/cfg:label"/>
                        </xsl:otherwise>
                    </xsl:choose>
                </a>
            </td>
            <td>
                <a title="see information about the status of the view">
                    <xsl:attribute name="href">
                        /lavoisier/status/jmx/domain/group/mbean[@name='<xsl:value-of select="$viewName"/>']
                    </xsl:attribute>
                    <xsl:choose>
                        <xsl:when test="$statusNode/attribute[@name='CurrentBuildStartDate']/value/text()!='null'">
                            <span class="badge badge-info">Currently building</span>
                        </xsl:when>
                        <xsl:when test="$statusNode/attribute[@name='LastException']/value/text()!='null'">
                            <span class="badge badge-warning">Exception raised</span>
                        </xsl:when>
                        <xsl:when test="$statusNode/attribute[@name='LastBuildDate']/value/text()='null'">
                            <span class="badge">Not built yet</span>
                        </xsl:when>
                        <xsl:otherwise>
                            <span class="badge badge-success">Successfully build</span>
                        </xsl:otherwise>
                    </xsl:choose>
                </a>
            </td>
            <td>
                <xsl:value-of select="cfg:info/cfg:description/text()"/>
            </td>
            <td>
                <xsl:value-of select="@path-template"/>
            </td>
            <td>
                <div class="btn-group">
                    <button class="btn btn-info dropdown-toggle" data-toggle="dropdown">types
                        <span class="caret"/>
                    </button>
                    <ul class="dropdown-menu">
                        <xsl:attribute name="aria-labelledby">drop_<xsl:value-of select="$viewName"/>
                        </xsl:attribute>
                        <xsl:call-template name="mimeList">
                            <xsl:with-param name="view">
                                <xsl:value-of select="$viewName"/>
                            </xsl:with-param>
                        </xsl:call-template>
                    </ul>
                </div>
            </td>
            <td>
                <a title="see code source of the view configuration">
                    <xsl:attribute name="href">
                        /lavoisier/configuration/cfg:config/cfg:view[@name='<xsl:value-of select="$viewName"/>']
                    </xsl:attribute>
                    view
                </a>
            </td>

        </tr>
    </xsl:template>

    <!-- mime as a dropdownlist -->
    <xsl:template name="mimeList">
        <xsl:param name="view"/>
        <xsl:for-each select="$mimeView/e:entries/mbean/attribute[@name='extensions']/value">
            <li>
                <a tabindex="-1">
                    <xsl:variable name="ext" select="substring-after(@default, '[')"/>
                    <xsl:if test="substring-before($ext, ',') != ''">
                        <xsl:attribute name="href">
                            /lavoisier/<xsl:value-of select="$view"/>?accept=<xsl:value-of
                                select="substring-before($ext, ',')"/>
                        </xsl:attribute>
                        <xsl:value-of select="substring-before($ext, ',')"/>
                    </xsl:if>
                    <xsl:if test="(substring-before($ext, ',') = '')">
                        <xsl:attribute name="href">
                            /lavoisier/<xsl:value-of select="$view"/>?accept=<xsl:value-of
                                select="substring-before($ext, ']')"/>
                        </xsl:attribute>
                        <xsl:value-of select="substring-before($ext, ']')"/>
                    </xsl:if>
                </a>
            </li>
        </xsl:for-each>
    </xsl:template>


</xsl:stylesheet>