<xsl:stylesheet version="1.0"  xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                xmlns:date="http://exslt.org/dates-and-times"
                extension-element-prefixes="date">
    <xsl:output method="html" encoding="utf-8" indent="yes" />

    <xsl:template match="/">
        <META HTTP-EQUIV="Refresh" CONTENT="10" />
        <html>
            <xsl:copy-of select="document('/lavoisier/head')"/>
            <body data-spy="scroll" data-target=".subnav" data-offset="50">
                <xsl:copy-of select="document('/lavoisier/navbar')"/>
                <div class="container fluid" style="margin-top:80px">
                    <table class="table table-bordered table-striped table-sorted table-hover">
                        <tr>
                            <th>View</th>
                            <th>Status - logs</th>							
                            <th>Last build date</th>
                            <th>Last Build Duration (s)</th>
                            <th>Last Build Size (Ko)</th>
                            <th>On going building (%)</th>
                        </tr>
                        <xsl:apply-templates/>
                    </table>
                </div>
            </body>
        </html>
    </xsl:template>

    <xsl:template match="mbean">
        <tr>
            <td>
                <xsl:value-of select="@name"/>
            </td>
            <td>
              <xsl:choose>
                    <xsl:when test="attribute[@name='CurrentBuildStartDate']/value/text()!='null'">
                        <table>
			                <tr><td><span class="badge badge-info">Currently building</span></td></tr>

			            </table>
                    </xsl:when>
                    <xsl:when test="attribute[@name='LastException']/value/text()!='null'">
			            <table>
			                <tr><td><span class="badge badge-warning">Exception raised</span></td></tr>
                            <tr><td><xsl:apply-templates select="attribute[@name='LastException']/value"/></td></tr>
			            </table>
                    </xsl:when>
                    <xsl:when test="attribute[@name='LastBuildDate']/value/text()='null'">
                        <span class="badge">Not built yet</span>
                    </xsl:when>
                    <xsl:otherwise>
                         <span class="badge badge-success">Successfully build</span>
                    </xsl:otherwise>
                </xsl:choose>
            </td>
            <xsl:apply-templates select="attribute[@name='LastBuildDate']"/>

            <td>
                <xsl:choose>
                    <xsl:when test="attribute[@name='LastBuildTime']!='null'">
                        <xsl:value-of select="round(attribute[@name='LastBuildTime']/value/text() div 1000)"/>
                    </xsl:when>
                    <xsl:otherwise>
                        <span class="label">Not Available</span>
                    </xsl:otherwise>
                </xsl:choose>
            </td>
            <td>
                <xsl:choose>
                    <xsl:when test="attribute[@name='LastBytesSize']!='null'">
                        <xsl:value-of select="round(attribute[@name='LastBytesSize']/value/text() div 1024)"/>
                    </xsl:when>
                    <xsl:otherwise>
                        <span class="label">Not Available</span>
                    </xsl:otherwise>
                </xsl:choose>
            </td>
            <td style="font-size:12px">
                <xsl:choose>
                    <xsl:when test="attribute[@name='CurrentBytesSize']/value/text()!='null' and attribute[@name='LastBytesSize']/value/text()!='null' ">
                        <xsl:variable name="percentage" select="round(attribute[@name='CurrentBytesSize']/value/text() div attribute[@name='LastBytesSize']/value/text() *100)"></xsl:variable>
                        <div class="progress">
                            <div class="bar" style="width: {$percentage}%;"></div>
                        </div>
                        <div>
                            <xsl:apply-templates select="attribute[@name='CurrentBuildStartDate']/value"/>
                        </div>
                    </xsl:when>
                    <xsl:otherwise>
                        <span class="label">Not Available</span>
                    </xsl:otherwise>
                </xsl:choose>
            </td>
        </tr>
    </xsl:template>

    <xsl:template match="attribute">
        <td style="font-size:12px">
            <xsl:apply-templates/>
        </td>
    </xsl:template>

    <xsl:template match="attribute[@type='java.util.Date']/value">
        <xsl:value-of select="substring-before(text(), 'CEST')"/>
    </xsl:template>
    <xsl:template match="value">
        <xsl:value-of select="text()"/>
    </xsl:template>

    <xsl:template match="*[namespace-uri()='http://software.in2p3.fr/lavoisier/entries.xsd' and local-name()='entries']">
        <xsl:choose>
            <xsl:when test="count(*) > 1">
                <ul>
                    <xsl:for-each select="*">
                        <li>
                            <xsl:apply-templates select="."/>
                        </li>
                    </xsl:for-each>
                </ul>
            </xsl:when>
            <xsl:otherwise>
                <xsl:apply-templates/>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>
    <xsl:template match="*[namespace-uri()='http://software.in2p3.fr/lavoisier/entries.xsd' and local-name()='entry']">
        <xsl:value-of select="text()"/>
    </xsl:template>

    <xsl:template match="*">
        <xsl:apply-templates/>
    </xsl:template>
</xsl:stylesheet>